@@ -24,36 +24,32 @@
 import dbus
 import os as _os
 
-from factory import CachedAccessibleFactory, AccessibleFactory
+from busutils import *
+
+from factory import AccessibleFactory
 from appevent import _ApplicationEventRegister, _NullApplicationEventRegister
 from deviceevent import _DeviceEventRegister, _NullDeviceEventRegister
-from desktop import *
-from cache import *
-from loop import *
+from cache import AccessibleCache
 
 from deviceevent import KEY_PRESSED_EVENT as _KEY_PRESSED_EVENT
 from deviceevent import KEY_RELEASED_EVENT as _KEY_RELEASED_EVENT
 
 from interfaces import ATSPI_REGISTRY_NAME as _ATSPI_REGISTRY_NAME
-from interfaces import ATSPI_DESKTOP_PATH as _ATSPI_DESKTOP_PATH
+from interfaces import ATSPI_ROOT_PATH as _ATSPI_ROOT_PATH
 from interfaces import ATSPI_DESKTOP as _ATSPI_DESKTOP
 
 __all__ = ["Registry",
-           "set_default_registry",
-           "MAIN_LOOP_GLIB",
-           "MAIN_LOOP_NONE",
-           "MAIN_LOOP_QT"]
+	   "MAIN_LOOP_GLIB",
+	   "MAIN_LOOP_NONE",
+	   "set_default_registry"]
 
 import gobject
-from dbus.mainloop.glib import DBusGMainLoop
-DBusGMainLoop (set_as_default=True)
-del DBusGMainLoop
 
 #------------------------------------------------------------------------------
 
 MAIN_LOOP_GLIB = 'GLib'
-MAIN_LOOP_NONE = 'None'
 MAIN_LOOP_QT   = 'Qt'
+MAIN_LOOP_NONE = 'None'
 
 #------------------------------------------------------------------------------
 
@@ -89,8 +85,8 @@ class Registry(object):
                 self.device_event_register = None
                 self.app_event_register = None
                 self.desktop = None
-                self.accessible_factory = None
-                self.main_loop = None
+
+		self.main_loop = gobject.MainLoop()
 
         def __call__(self):
                 """
@@ -119,59 +115,34 @@ class Registry(object):
 
                 @param app_name: D-Bus name of the application to connect to when not using the registry daemon.
                 """
-                connection = dbus.SessionBus()
 
-                # Set up the main loop
+
+                # Set up the cache
+		cache = None
                 if main_loop_type == MAIN_LOOP_GLIB:
-                        loop   = GObjectMain()
-                        proxy  = dbus.connection.ProxyObject
-                        #proxy  = GObjectProxy
-                elif main_loop_type == MAIN_LOOP_NONE:
-                        loop    = NullMain()
-                        proxy  = dbus.connection.ProxyObject
-                else:
-                        raise Exception ("Unknown main loop specified")
+                                cache = AccessibleCache (app_name)
+
+		factory = AccessibleFactory(cache)
 
                 # Set up the device event controllers
+                _connection = SyncAccessibilityBus ()
+                _bus_object = _connection.get_object("org.freedesktop.DBus", "/org/freedesktop/DBus")
+
                 if app_name:
-                        devreg = _NullDeviceEventRegister()
-                        appreg = _NullApplicationEventRegister()
+                        self.device_event_register = _NullDeviceEventRegister()
+                        seld.app_event_register    = _NullApplicationEventRegister()
+
+			name = _bus_object.GetNameOwner (app_name)
+                        self.desktop = factory (name, _ATSPI_ROOT_PATH, _ATSPI_DESKTOP)
                 else:
-                        devreg = _DeviceEventRegister(connection)
-                        appreg = _ApplicationEventRegister(connection)
+                        self.device_event_register = _DeviceEventRegister()
+                        self.app_event_register    = _ApplicationEventRegister(factory)
 
-                # Set up the cache / desktop and accesible factories.
-                if main_loop_type == MAIN_LOOP_GLIB:
-                        if app_name:
-                                cache = AccessibleCache(appreg, connection, app_name)
-                        else:
-                                cache = ApplicationCache(appreg, connection)
-                        appreg.setCache (cache)
-                        factory = CachedAccessibleFactory (cache, connection, proxy)
-                        cache.set_factory(factory)
-                        desktop = CachedDesktop (cache, factory)
+			name = _bus_object.GetNameOwner (_ATSPI_REGISTRY_NAME)
+                        self.desktop = factory (name, _ATSPI_ROOT_PATH, _ATSPI_DESKTOP)
 
-                elif main_loop_type == MAIN_LOOP_NONE:
-                        factory = AccessibleFactory(connection, proxy)
-                        if app_name:
-                                desktop = TestDesktop (connection, app_name, factory)
-                        else:
-                                desktop = Desktop (connection, factory)
-
-                else:
-                        raise Exception ("Unknown main loop specified")
-       
-                appreg.setFactory (factory)
-
-                # Create the registry object
                 self.has_implementations = True
 
-                self.device_event_register = devreg
-                self.app_event_register = appreg
-                self.desktop = desktop
-                self.accessible_factory = factory
-                self.main_loop = loop
-
         def _set_default_registry (self):
                 self._set_registry (MAIN_LOOP_GLIB)
 
@@ -201,7 +172,7 @@ class Registry(object):
                 """
                 if not self.has_implementations:
                         self._set_default_registry ()
-                self.main_loop.stop()
+                self.main_loop.quit()
 
         def getDesktopCount(self):
                 """
